#ifndef GLPLOT_H

#include <qthread.h>
#include <qgl.h>
#include <qdatetime.h>
#include "Frame.h"
#include "vectormath.h"
#include <qfiledialog.h>

class Camera {
  public:
    float phi, th, dist;
    float aspect, fov;
    
    Camera();
    void setProjMatrix();

};

struct oscPoint {
  float val;
  int time;
};

#define GLP_NUM_DRAW_MODES 3
class GLPlot : public QGLWidget {
  Q_OBJECT
  public:
    GLPlot( QWidget *parent );
    void newFrame(Frame);
    
    void savePlot(QString filename, bool stream);
    float* getPlotAvg(int *width);
    void draw2DOverlay(float* data, int width);
    
    void setDrawMode(int);
    void setShowInput(bool);
    void setWireframe(bool);
    void setUseImage(bool);
    void setLight(bool);
    void setLightPos(float angle);
    void setOscLength(int length);
    void resizeOscilloscope(int newSize);
    void setPPU(float);
    void setDrawEnabled(bool);
    void setStoreEnabled(bool);
    void setShowGrid(bool);
  
  protected:
    void initializeGL();
    void resizeGL(int, int);
    void paintGL();
    void mouseMoveEvent(QMouseEvent * e);
    void mousePressEvent( QMouseEvent * e);
    void mouseReleaseEvent( QMouseEvent * e);
    void wheelEvent(QWheelEvent* e);
  
  private:
    bool drawEnabled;
    bool storeEnabled;
    QString storeFile;
    Frame plotData;
    QMutex plotDataMutex;
    
    void doVertex(Vector);
    
    float* vAverage;
    float* overlay;
    
    float pxPerUnit;
    
    Camera cam;
    Vector scale[GLP_NUM_DRAW_MODES];
    Vector translate[GLP_NUM_DRAW_MODES];
    Vector mousePress, mousePos;
    int mouseButton;
    
    Vector lightPos;
    bool light;
    bool wireframe;
    bool useImage;
    int drawMode;
    bool showInput;
    bool showGrid;
    
    int oscLength;
    oscPoint* oscData;
    int oscPointer;
    
    QTime dTime;
    
    void init2D();
    void init3D();
    void drawInput();
    void changeScale(int, float, float);
    void recomputeAverage();
    void refreshOscilloscope();
    void drawGrid();
    void drawCrosshair();
    void drawPlot();
    void drawScope();
    void setupLighting();
    void drawSurface();
    void drawWireframe();
};



#define GLPLOT_H
#endif
