#include "Handle.h"
#include <qbitmap.h>
#include <qpainter.h>

Vector qPtVec(const QPoint p) {
  return Vector(p.x(), p.y());
}

QPoint vecQPt(const Vector v) {
  return QPoint((int)v[0], (int)v[1]);
}


// Handle widget

Handle::Handle( QWidget *parent, QPaintDevice* p ) : QLabel( parent ) {
  paintDevice = p;
  id = 0;
  init();
}

Handle::Handle( QWidget *parent, QPaintDevice* p, int i ) : QLabel( parent ) {
  paintDevice = p;
  id = i;
  init();
}

void Handle::init() {
  setMask(QBitmap(width(), height(), TRUE));
  setFixedSize(7, 7);
  setPaletteBackgroundColor( QColor( 255, 255, 255 ) );
}

void Handle::setId(int i) {
  id = i;
}

void Handle::setPos(const QPoint p) {
  move(p - QPoint(width() / 2, height() / 2));
}

void Handle::setAngle(float a) {
  angle = a;
}

void Handle::setPaintDevice(QPaintDevice* p) {
  paintDevice = p;
}

void Handle::mouseMoveEvent ( QMouseEvent * e ) {
  if( ! parentWidget()->isEnabled() )
    return;
  e->accept();
  QPoint newPos(e->globalPos() - pressMousePos);
  
  // Check parent boundaries
  int w = parentWidget()->width() - width();
  int h = parentWidget()->height() - height();
  int w2 = width() / 2;
  int h2 = height() / 2;
  if( newPos.x() < -w2 )
    newPos.setX(-w2);
  if( newPos.y() < -h2 )
    newPos.setY(-h2);
  if( newPos.x() > w+w2 )
    newPos.setX(w+w2);
  if( newPos.y() > h+h2 )
    newPos.setY(h+h2);
  
  // Move the handle
  if( newPos != pos() ) {
    move(newPos);
    emit posChanged(id);
  }
  
}

void Handle::mousePressEvent ( QMouseEvent * e ) {
  if( ! parentWidget()->isEnabled() )
    return;
  e->accept();
  pressMousePos = e->globalPos() - pos();
}

void Handle::paintEvent( QPaintEvent * ) {
//  drawWidget(paintDevice);
}

void Handle::drawWidget(QPaintDevice* p) {
  if( p == NULL )
    return;
  QPainter pntr(p);
  
  pntr.setPen(QPen(QColor(200, 200, 255), 1));
  pntr.setRasterOp(Qt::XorROP);
  pntr.translate(x(), y());
  pntr.drawRect(0, 0, width(), height());
  pntr.resetXForm();
}

QPoint Handle::center() {
  return QPoint(pos() + QPoint(width() / 2, height() / 2));
}
