#include "HandleLine.h"

#include <qbitmap.h>
#include <qpainter.h>


HandleLine::HandleLine( QWidget *parent, QPaintDevice* p ) : QWidget(parent) {
  paintDevice = p;
  setFixedSize(0, 0);
  //hide();
  //setBackgroundOrigin(QWidget::ParentOrigin);
  setMask(QBitmap(width(), height(), TRUE));
  
  pts = new Handle*[4];
  for( int i=0; i<4; i++ ) {
    pts[i] = new Handle(parent, paintDevice, i);
    
    connect(pts[i], SIGNAL(posChanged(int)), this, SLOT(handleMoved(int)));
  }
  pts[0]->setPos(QPoint(50, 50));
  pts[1]->setPos(QPoint(100, 50));
  pts[2]->setPos(QPoint(75, 50));
  pts[3]->setPos(QPoint(75, 56));
  regWidth = 50;
  regHeight = 1;
  regAngle = 0;
}

void HandleLine::setPaintDevice(QPaintDevice* p) {
  paintDevice = p;
  for( int i=0; i<4; i++ ) {
    pts[i]->setPaintDevice(p);
  }
}

void HandleLine::paintEvent( QPaintEvent * ) {
  drawWidget(paintDevice);
}

void HandleLine::drawWidget(QPaintDevice* w) {
  if( w == NULL || ! isShown() )
    return;
  QPainter pntr(w);
  Vector p[4];
  for( int i=0; i<4; i++  ) {
    p[i] = qPtVec(pts[i]->pos());
  }
  pntr.setPen(QPen(QColor(200, 200, 255), 1));
  pntr.setRasterOp(Qt::XorROP);
  pntr.translate(pts[0]->center().x(), pts[0]->center().y());
  pntr.rotate(-regAngle * R2D);
  pntr.drawRect(0, 0, (int)regWidth, (int)regHeight);
  pntr.resetXForm();
  for( int i=0; i<4; i++ ) {
    pts[i]->drawWidget(w);
  }
}


void HandleLine::handleMoved(int id) {
  Vector p[4];
  for( int i=0; i<4; i++  ) {
    p[i] = qPtVec(pts[i]->center());
  }
  
  if( id < 2 ) {
    Vector diff = p[1] - p[0];
    
    regWidth = diff.mag();
    if( regWidth == 0 ) {
	p[id] += p[1-id]+Vector(1,0);
	pts[id]->setPos(vecQPt(p[id]));
	diff = p[1] - p[0];
	regWidth = 1.0;
    }
    regAngle = diff.angle(Vector(1, 0, 0)) * sign(((p[1]-p[0]) * Vector(1))[2]);
    for( int i=0; i<4; i++ ) {
      pts[i]->setAngle(regAngle);
    }
    pts[2]->setPos(vecQPt((p[0] + p[1]) * 0.5));
    pts[3]->setPos(vecQPt(p[2] + ((Vector(0, 0, 1) * diff).norm() * (regHeight + 5))));
  }
  else if( id == 2 ) {
    Vector diff = p[2] - ((p[1] + p[0]) * 0.5);
    pts[0]->setPos(vecQPt(diff + p[0]));
    pts[1]->setPos(vecQPt(diff + p[1]));
    pts[3]->setPos(vecQPt(diff + p[3]));
  }
  else if( id == 3 ) {
    Vector diff = p[1] - p[0];
    Vector hd = p[3]-p[2];
    p[3] -= hd.proj(diff);
    regHeight = (p[2] - p[3]).mag() - 5;
    if( regHeight < 1 || ((p[0]-p[2])*(p[3]-p[2]))[2] > 0 ) {
      regHeight = 1;
    }
    p[3] = p[2] - ((Vector(0, 0, 1) * (p[0]-p[2])).norm() * (regHeight+5));
    
    pts[3]->setPos(vecQPt(p[3]));
  }
//  repaint();
  emit regionChanged();
}

QPoint HandleLine::pos() {
  return pts[0]->center();
}
QSize HandleLine::size() {
  return QSize((int)regWidth, (int)regHeight);
}
float HandleLine::angle() {
  return regAngle;
}
