#include "HandleRect.h"

#include <qbitmap.h>
#include <qpainter.h>

HandleRect::HandleRect( QWidget *parent, QPaintDevice* p ) : QWidget(parent) {
  paintDevice = p;
  setFixedSize(0, 0);
  setMask(QBitmap(width(), height(), TRUE));
  
  pts = new Handle*[4];
  for( int i=0; i<4; i++ ) {
    pts[i] = new Handle(parent, paintDevice, i);
    connect(pts[i], SIGNAL(posChanged(int)), this, SLOT(handleMoved(int)));
  }
  pts[0]->setPos(QPoint(0, 0));
  pts[1]->setPos(QPoint(1, 0));
  pts[2]->setPos(QPoint(0, 1));
  pts[3]->setPos(QPoint(1, 1));
  regWidth = 1;
  regHeight = 1;
}

void HandleRect::setRegion(QRect r) {
  pts[0]->setPos(r.topLeft());
  handleMoved(0);
  pts[1]->setPos(r.topRight());
  handleMoved(1);
  pts[2]->setPos(r.bottomLeft());
  handleMoved(2);
  pts[3]->setPos(r.bottomRight());
  handleMoved(3);
//   printf("Changed region to %d, %d - %d, %d (%dx%d)\n", pts[0]->center().x(), pts[0]->center().y(), pts[3]->center().x(), pts[3]->center().y(), r.width(), r.height());
  emit regionChanged();
}

void HandleRect::setPaintDevice(QPaintDevice* p) {
  paintDevice = p;
  for( int i=0; i<4; i++ ) {
    pts[i]->setPaintDevice(p);
  }
}

void HandleRect::paintEvent( QPaintEvent * ) {
  drawWidget(paintDevice);
}

void HandleRect::drawWidget(QPaintDevice* w) {
  if( w == NULL || ! isShown() )
    return;
  QPainter pntr(w);
  Vector p[4];
  for( int i=0; i<4; i++  ) {
    p[i] = qPtVec(pts[i]->pos());
  }
  pntr.setPen(QPen(QColor(200, 200, 255), 1));
  pntr.setRasterOp(Qt::XorROP);
  pntr.translate(pts[0]->center().x(), pts[0]->center().y());
  pntr.drawRect(0, 0, (int)regWidth, (int)regHeight);
  pntr.resetXForm();
  for( int i=0; i<4; i++ ) {
    pts[i]->drawWidget(w);
  }
}


void HandleRect::handleMoved(int id) {
  Vector p[4];
  for( int i=0; i<4; i++  ) {
    p[i] = qPtVec(pts[i]->center());
  }
  regWidth = p[1][0] - p[0][0] + 1;
  regHeight = p[2][1] - p[0][1] + 1;
  
  if( id == 0 ) {
    pts[1]->setPos(QPoint((int)p[1][0], (int)p[0][1]));
    pts[2]->setPos(QPoint((int)p[0][0], (int)p[2][1]));
  }
  else if( id == 1 ) {
    pts[0]->setPos(QPoint((int)p[0][0], (int)p[1][1]));
    pts[3]->setPos(QPoint((int)p[1][0], (int)p[3][1]));
  }
  else if( id == 2 ) {
    pts[0]->setPos(QPoint((int)p[2][0], (int)p[0][1]));
    pts[3]->setPos(QPoint((int)p[3][0], (int)p[2][1]));
  }
  else if( id == 3 ) {
    pts[1]->setPos(QPoint((int)p[3][0], (int)p[1][1]));
    pts[2]->setPos(QPoint((int)p[2][0], (int)p[3][1]));
  }
  
  
//  repaint();
  emit regionChanged();
}

QPoint HandleRect::pos() {
  return pts[0]->center();
}
QSize HandleRect::size() {
  return QSize(regWidth, regHeight);
}
