#include <qmessagebox.h>

#include "VideoDisplayArea.h"

// This really should not be here.
#include <uEye.h>



VideoDisplayArea::VideoDisplayArea(QWidget* parent) : QLabel(parent) {
//  setPaletteBackgroundColor(QColor(0,0,0));
  setBackgroundMode(Qt::NoBackground);
  setFixedSize(0, 0);
  newFrameReady = false;
  vidFrame = NULL;
  vidData = NULL;
  displayFrame = NULL;
  vidPosX = vidPosY = 0;
  
  for ( int i=0; i<256; i++)
    m_table[i] = qRgb(i,i,i);
  
  plotRegion = new HandleLine(this, displayFrame);
  connect(plotRegion, SIGNAL(regionChanged()), this, SLOT(OnPlotRegionChange()));
  
  cropRegion = new HandleRect(this, displayFrame);
  connect(cropRegion, SIGNAL(regionChanged()), this, SLOT(OnCropRegionChange()));
  cropRegion->setRegion(QRect(QPoint(0, 0), QSize(100, 100)));
  
  
}

void VideoDisplayArea::OnCropRegionChange() {
  update();
  emit cropRegionChanged();
}

void VideoDisplayArea::OnPlotRegionChange() {
  update();
  emit plotRegionChanged();
//  emit plotDataChanged();
}

void VideoDisplayArea::setPlotRegionShown(bool b) {
  plotRegion->setShown(b);
  plotRegion->setEnabled(b);
}

void VideoDisplayArea::setCropRegionShown(bool b) {
  cropRegion->setShown(b);
  cropRegion->setEnabled(b);
}

void VideoDisplayArea::setCropRegion(QRect r) {
  cropRegion->setRegion(r);
}

QRect VideoDisplayArea::plotRegionRect() {
  return QRect(plotRegion->pos(), plotRegion->size());
}

float VideoDisplayArea::plotRegionAngle() {
  return plotRegion->angle();
}

QRect VideoDisplayArea::cropRegionRect() {
  return QRect(cropRegion->pos(), cropRegion->size());
}

void VideoDisplayArea::newVideoFrame(uchar* buffer, int x, int y, int w, int h, int bpp, int mode) {
  if( h<1 || w<1 || bpp<1 || buffer == NULL )
    return;
  vidPosX = x;
  vidPosY = y;
  
  QRgb *pRgb  = NULL;
  int entries = 0;
  if ( mode == IS_SET_CM_Y8 ) {
    pRgb   = m_table;
    entries= 256;
  }
  QImage *image = new QImage( (uchar*)buffer, w, h, bpp, pRgb, entries, QImage::IgnoreEndian );
  QImage* old = vidData;
  vidData = image;

  if( old != NULL ) {
    delete old;
  }
  newFrameReady = TRUE;
  update();
//  emit plotDataChanged();
}

void VideoDisplayArea::drawImage() {
  if( displayFrame == NULL )
    return;
  
  displayFrame->fill(QColor(0,0,0));
  if( newFrameReady ) {
    delete vidFrame;
    vidFrame = new QPixmap(*vidData);
    newFrameReady = FALSE;
  }
  if( vidFrame != NULL ) {
    bitBlt(displayFrame, vidPosX, vidPosY, vidFrame);
  }
  
  plotRegion->drawWidget(displayFrame);
  cropRegion->drawWidget(displayFrame);
  bitBlt(this, 0, 0, displayFrame);      // Draw pixmap
}

void VideoDisplayArea::paintEvent(QPaintEvent *e) {
  drawImage();
}

void VideoDisplayArea::setSize(QSize s) {
  setFixedSize(s);
  delete displayFrame;
  displayFrame = new QPixmap(s);
  cropRegion->setRegion(QRect(QPoint(0,0), s));
  cropRegion->setPaintDevice(displayFrame);
  plotRegion->setPaintDevice(displayFrame);
}

void VideoDisplayArea::saveImage(QString filename) {
  FILE* fd = fopen(filename, "w");
  if( fd == NULL ) {
    QMessageBox::warning( this, "Bummer, man.", "Could not open file for writing!", QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);
    return;
  }
  fclose(fd);
  
  if( vidData == NULL ) {
    QMessageBox::warning( this, "Bummer, man.", "No video image to save!", QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);
    return;
  }
  vidData->save(filename, "PNG");
}
