#ifndef VIDEODISPLAYAREA_H

#include <qlabel.h>
#include <qpixmap.h>
#include <qimage.h>

#include "HandleLine.h"
#include "HandleRect.h"

class VideoDisplayArea : public QLabel {
  Q_OBJECT
  public:
    VideoDisplayArea( QWidget *parent );
    void newVideoFrame(uchar* buffer, int x, int y, int w, int h, int bpp, int mode);
    void setSize(QSize);
    void setPlotRegionShown(bool);
    void setCropRegionShown(bool);
    void setCropRegion(QRect);
    QRect plotRegionRect();
    float plotRegionAngle();
    QImage* plotData();
    QRect cropRegionRect();
    
    void saveImage(QString s);
    
    
  public slots:
    void OnCropRegionChange();
    void OnPlotRegionChange();
    
  signals:
    void cropRegionChanged();
    void plotRegionChanged();
    
  protected:
    void paintEvent( QPaintEvent * );

  private: 
    HandleLine* plotRegion;
    HandleRect* cropRegion;
    int vidPosX, vidPosY;
    QImage* vidData;
    QPixmap *vidFrame, *displayFrame;
  // Colorindex table for monochrom pictures
    QRgb m_table[256];  
    void drawImage();
    bool newFrameReady;
    
    
};

#define VIDEODISPLAYAREA_H
#endif
