/***************************************************************************
 *   Copyright (C) 2005                                                    *
 *   Luke Campagnola                                                       *
 *   Colorado School of Mines                                              *
 *   lcampagn@mines.edu                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "dlgmainview.h"
#include "dlgproperties.h"
#include "mathFunctions.h"


// -------------------  Constructor  -----------------------
//
DlgMainView::DlgMainView(QWidget* parent, const char* name, WFlags fl)
: MainView(parent,name,fl)
{
  clock.start();
  ZeroMemory( m_Images, sizeof(m_Images) );  
  checkUseMemoryboard->setChecked(FALSE);
    

  m_hCamera   = NULL;
  m_nWidth  = 0;
  m_nHeight  = 0;
  
    
  m_nColorMode = IS_SET_CM_Y8;
  m_nBitsPP = 8;
  
  char str[255];
  DWORD ver = is_GetDLLVersion();
  sprintf(str, "API-Version: %d.%d.%d", ver>>24, ver>>16&0xff, ver&0xffff );
  textStatus->setText( str );
  
  m_bLive = FALSE;
  
  m_pLastBuffer = NULL;
  m_displayBuffer = NULL;
  m_groundBuffer = NULL;
  m_averageBuffer = NULL;
  m_finalBuffer = NULL;
  m_bSetGroundFrame = false;

  m_nAvgConstant = 1.0;
  
  ImageThread::start();
  
  m_bRunEventThread = TRUE;
  EventThread::start();
  
  is_EnableEvent( NULL, IS_SET_EVENT_NEW_DEVICE );
  is_EnableEvent( NULL, IS_SET_EVENT_REMOVAL );  
  
  spinAverageConstant->setPrecision( 4 );
  spinAverageConstant->setMaxValue( 1 );
  spinBrightness->setLineStep(1.0);
  spinBrightness->setMaxValue(100.0);
  spinBrightness->setMinValue(-100.0);

  videoArea = new VideoDisplayArea(frameVideo);
  connect(videoArea, SIGNAL(cropRegionChanged()), this, SLOT(OnActiveRegionChange()));
  connect(videoArea, SIGNAL(plotRegionChanged()), this, SLOT(OnPlotRegionChange()));
  plotArea = new GLPlot(framePlot);
  plotArea->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
  plotArea->setFixedSize(framePlot->width(), framePlot->height());
/*  connect(videoArea, SIGNAL(plotDataChanged()), this, SLOT(OnPlotDataChanged()));*/
  connect(pushUpdateCropRegion, SIGNAL(clicked()), this, SLOT(OnBtnUpdateCropRegion()));

  plotThread = new PlotThread(plotArea);
//  plotThread->start(QThread::LowestPriority);
  plotThread->start();
  OnPlotRegionChange();
}

// -------------------  Destructor  -----------------------
//
DlgMainView::~DlgMainView()
{
  if (m_bThreadRun)
  {
    m_bThreadRun = FALSE;
    if ( ImageThread::wait(250) == FALSE )
    {
      ImageThread::terminate();
    }
  }

  if (m_bRunEventThread)
  {
    m_bRunEventThread = FALSE;
    if ( EventThread::wait(250) == FALSE )
      EventThread::terminate();
  }
  
  if ( m_hCamera )
    is_ExitCamera( m_hCamera );
  m_hCamera = NULL;
}


// -------------------  closeEvent  -----------------------
//
void DlgMainView::closeEvent ( QCloseEvent * e )
{
  m_bLive = FALSE;
  is_StopLiveVideo( m_hCamera, IS_WAIT );
  m_bThreadRun = FALSE;
  m_bRunEventThread = FALSE;
  msleep(250);
  
  if ( m_hCamera )
    is_ExitCamera( m_hCamera );
  m_hCamera = NULL;
  
  e->accept ();
}


// -------------------  timerEvent  -----------------------
//
void DlgMainView::timerEvent( QTimerEvent *e )
{
}


// -------------------  DlgMainViewpaintEvent  -----------------------
//
void DlgMainView::paintEvent ( QPaintEvent *e )
{
//   if ( m_displayBuffer  )
//     DrawImage( m_displayBuffer );

  m_bReady = TRUE;
}





// -------------------  OnBtnInituEye  -----------------------
//
void DlgMainView::OnBtnInituEye() 
{
  char str[512];
  if ( m_hCamera )
    is_ExitCamera(m_hCamera);
  
  m_hCamera = 0;  // open next camera
  if ( is_InitCamera( &m_hCamera, 0 ) != IS_SUCCESS )
  {
    m_hCamera   = NULL;
    QMessageBox::information ( 0, "Error !!!", "Open camera failed", 0 );
  }
  else 
  {  
    if ( is_GetCameraInfo( m_hCamera, &m_CamInfo ) != IS_SUCCESS )
      QMessageBox::information ( 0, "Error !!!", "GetCameraInfo failed", 0 );
    else if ( is_GetSensorInfo( m_hCamera, &m_SensorInfo ) != IS_SUCCESS )
      QMessageBox::information ( 0, "Error !!!", "GetSensorInfo failed", 0 );
    else
    {
      m_nWidth  = m_SensorInfo.nMaxWidth;
      m_nHeight = m_SensorInfo.nMaxHeight;
      videoArea->setSize(QSize(m_nWidth, m_nHeight));
/*      pixmapImage->setFixedSize(m_nWidth, m_nHeight);
      activeRegion->setRegion(QRect(QPoint(0, 0), QSize(m_nWidth, m_nHeight)));*/
/*      updateCropRegion();*/
      BOARDINFO boardInfo;
      is_GetBoardInfo( m_hCamera, &boardInfo );
      sprintf(str, "Sensor: %s    -   S/N: %s\n", m_SensorInfo.strSensorName, boardInfo.SerNo );
      textStatus_2->setText(str);
      sprintf(str, "%dx%d\n", m_nWidth, m_nHeight );
      textStatus_3->setText(str);  
      OnRadioColorChange(QButton::On);    
    }
    
    pushInit->setEnabled( FALSE );
    pushExit->setEnabled( TRUE );
    pushFreeze->setEnabled( TRUE );
    pushLive->setEnabled( TRUE );
    pushProperties->setEnabled( TRUE );    
    
    BOOL bConnected;
    is_IsMemoryBoardConnected( m_hCamera, &bConnected );
    if ( !bConnected )
      checkUseMemoryboard->setEnabled( FALSE );
      
    is_EnableEvent( m_hCamera, IS_SET_EVENT_FRAME );
  }
  
  m_pLastBuffer = NULL;
}


// -------------------  OnBtnExituEye  -----------------------
//
void DlgMainView::OnBtnExituEye()
{
  if ( m_hCamera )
  {
    if ( m_bLive )
      OnBtnLive();
    
    is_ExitCamera( m_hCamera );

    m_hCamera   = NULL;

    pushInit->setEnabled( TRUE );
    pushExit->setEnabled( FALSE );
    pushFreeze->setEnabled( FALSE );
    pushLive->setEnabled( FALSE );
    pushProperties->setEnabled( FALSE );

    textStatus_2->setText("");
    textStatus_3->setText("S/N:\n");  
    
  }
  else
    QMessageBox::information ( 0, "Error !!!", "Camera not open", 0 );
    
  m_pLastBuffer = NULL;
}



// -------------------  OnBtnSnapshot  -----------------------
//
void DlgMainView::OnBtnSnapshot()
{
  INT nRet, dummy; 
  char *pMem, *pLast;
  
  if ( m_hCamera )
  {
    if ( !m_bLive )
    {
      if ( checkUseMemoryboard->isChecked() )
        nRet = is_MemoryFreezeVideo( m_hCamera, 0, IS_WAIT );
      else
        nRet = is_FreezeVideo( m_hCamera, IS_WAIT );
      if ( nRet != IS_SUCCESS )
        QMessageBox::information ( 0, "Error !!!", "FreezeVideo failed", 0 );
      else
      {
        is_GetActSeqBuf(m_hCamera, &dummy, &pMem, &pLast );
        m_pLastBuffer = pLast;
        update();
      }  
    }
  }
  else
    QMessageBox::information ( 0, "Error !!!", "Camera not open", 0 );
}




// -------------------  OnBtnLive  -----------------------
//
void DlgMainView::OnBtnLive()
{    
  if ( !m_bLive )
  {  
    is_SetMemoryMode( m_hCamera, IS_MEMORY_MODE_DISABLE, 0 );
    checkUseMemoryboard->setChecked(FALSE);
    pushLive->setText("Stop Live");
    pushFreeze->setEnabled( FALSE );
    //pushProperties->setEnabled( FALSE );
    checkUseMemoryboard->setEnabled( FALSE );
    radioMonochrome->setEnabled( FALSE );
    radioColor->setEnabled( FALSE );
    OnBtnUpdateCropRegion();
    m_bLive = TRUE;
  }
  else
  {
    m_bLive = FALSE;
    BOOL bFinished = FALSE;
    int nTimeout = 0;
    while ( !bFinished && nTimeout++<100 )
    {
      is_IsVideoFinish( m_hCamera, &bFinished );
      msleep(10);
    }
    
    pushLive->setText("Live");
    pushFreeze->setEnabled( TRUE );
    pushProperties->setEnabled( TRUE );
    BOOL bConnected;
    is_IsMemoryBoardConnected( m_hCamera, &bConnected );
    if ( bConnected )
      checkUseMemoryboard->setEnabled( TRUE );

    radioMonochrome->setEnabled( TRUE );
    radioColor->setEnabled( TRUE );
    OnCheckUsememoryBoard();
  }
}

  

// -------------------  OnBtnSaveToFile  -----------------------
//
void DlgMainView::OnBtnSaveToFile()
{
  if ( m_hCamera )
  {
    Preview* p = new Preview;  
    QFileDialog* fd = new QFileDialog(  "/home",
                                "Images (*.png *.xpm *.jpg)",
                                this,
                                "open file dialog"
                                "Choose a file" );
    fd->setMode(QFileDialog::AnyFile);
    fd->setContentsPreviewEnabled( TRUE );
    fd->setContentsPreview( p, p );
    fd->setPreviewMode( QFileDialog::Contents );
    
    QString filter("Image files (*.bmp);;"  
                      "Any files (*)");      
    fd->setFilters( filter );

    //fd->show();
    QString fileName;
    if ( fd->exec() == QDialog::Accepted )
      fileName = fd->selectedFile();
    
    if ( !fileName.isEmpty() )
      is_SaveImageMem( m_hCamera, (char*)fileName.latin1(), m_Images[0].pBuf, m_Images[0].nImageID );
  }
  else
    QMessageBox::information ( 0, "Error !!!", "Camera not open", 0 );
}




// -------------------  OnBtnProperties  -----------------------
//
void DlgMainView::OnBtnProperties()
{
  // Properties dialog
  DlgProperties dlgProperties(m_hCamera, m_CamInfo, m_SensorInfo, this, 0, TRUE);
  dlgProperties.exec();
}
 


// -------------------  OnBtnProperties  -----------------------
//
void DlgMainView::OnRadioColorChange(int state)
{
  int bitsPixel = 0;
  if (state == QButton::On )
  {
    if ( radioMonochrome->isChecked() ) {
      m_nColorMode = IS_SET_CM_Y8;
      m_nBitsPP = 8;
    }
    else {
      m_nColorMode = IS_SET_CM_RGB32;
      m_nBitsPP = 32;
    }
        
    if ( !_AllocImages() )
      QMessageBox::information ( 0, "Error !!!", "Memory allocation failed", 0 );    
    else if ( is_SetImageSize( m_hCamera, m_nWidth, m_nHeight ) != IS_SUCCESS )
      QMessageBox::information ( 0, "Error !!!", "SetImageSize failed", 0 );
    else if ( is_SetColorMode( m_hCamera, m_nColorMode ) != IS_SUCCESS )
      QMessageBox::information ( 0, "Error !!!", "SetColorMode failed", 0 );
  }
}



// --------------------------- _GetImageID ------------------------------------
//
void DlgMainView::OnCheckUsememoryBoard()
{
  if ( checkUseMemoryboard->isChecked() )
    is_SetMemoryMode( m_hCamera, 1, 0 );
  else
    is_SetMemoryMode( m_hCamera, IS_MEMORY_MODE_DISABLE, 0 );
}


// --------------------------- _GetImageID ------------------------------------
//
INT DlgMainView::_GetImageID( char* pbuf )
{
  if ( !pbuf )
    return 0;

  for ( int i=0; i<sizeof(m_Images)/sizeof(m_Images[0]); i++)
    if ( m_Images[i].pBuf == pbuf )
      return m_Images[i].nImageID; 
  
  return 0;
}



// --------------------------- _GetImageNum ------------------------------------
//
INT DlgMainView::_GetImageNum( char* pbuf )
{
  if ( !pbuf )
    return 0;

  for ( int i=0; i<sizeof(m_Images)/sizeof(m_Images[0]); i++)
    if ( m_Images[i].pBuf == pbuf )
      return m_Images[i].nImageSeqNum;
  
  return 0;
}


// --------------------------- _EmptyImages ------------------------------------
//
void DlgMainView::_EmptyImages()
{
  m_pLastBuffer = NULL;
  for ( int i=0; i<sizeof(m_Images)/sizeof(m_Images[0]); i++) 
    if ( m_Images[i].pBuf )
    {
      is_LockSeqBuf(m_hCamera, m_Images[i].nImageSeqNum, m_Images[i].pBuf );
      ZeroMemory(m_Images[i].pBuf, m_Images[i].nBufferSize);
      is_UnlockSeqBuf(m_hCamera, m_Images[i].nImageSeqNum, m_Images[i].pBuf );
    }
}



// --------------------------- _AllocImages ------------------------------------
//
BOOL DlgMainView::_AllocImages()
{
//  INT nBitsPixel = (m_nColorMode == IS_SET_CM_Y8 ? 8 : 32 );
  
  m_pLastBuffer = NULL;
  
  is_ClearSequence(m_hCamera);
  for ( int i=0; i<sizeof(m_Images)/sizeof(m_Images[0]); i++ )
  {
    if ( m_Images[i].pBuf )
    {
      is_FreeImageMem( m_hCamera, m_Images[i].pBuf, m_Images[i].nImageID );
      m_Images[i].pBuf = NULL;
      m_Images[i].nImageID = 0;
    }
      
    if ( is_AllocImageMem( m_hCamera, m_nWidth, m_nHeight, m_nBitsPP, &m_Images[i].pBuf, &m_Images[i].nImageID ) != IS_SUCCESS )
      return FALSE;
    if ( is_AddToSequence( m_hCamera, m_Images[i].pBuf, m_Images[i].nImageID ) != IS_SUCCESS )
      return FALSE;
    
    m_Images[i].nImageSeqNum = i + 1;
    m_Images[i].nBufferSize = m_nWidth * m_nHeight * m_nBitsPP/8;
  }
  //is_SetImageMem( m_hCamera, m_Images[0].pBuf, m_Images[0].nImageID );

  return TRUE;
}



// --------------------------- RunEventThread ------------------------------------
//
void DlgMainView::RunEventThread()
{
  while ( m_bRunEventThread )
  {
    if ( is_WaitEvent( NULL, IS_SET_EVENT_NEW_DEVICE, 100 ) == IS_SUCCESS )
      printf("New Device detected\n");
    if ( is_WaitEvent( NULL, IS_SET_EVENT_REMOVAL, 100 ) == IS_SUCCESS )
      printf("Device Removal detected\n");
  }
  m_bRunEventThread = FALSE;
}





void DlgMainView::resetVideo() {
  
  
  int newBufSize = m_nWidth * m_nHeight * m_nBitsPP / 8;
  if( m_nBufSize != newBufSize && newBufSize > 0 ) {
    if( m_displayBuffer != NULL )
     delete [] m_displayBuffer;
    m_displayBuffer = new char[newBufSize];
    m_nDisplayWidth = m_nWidth;
    m_nDisplayHeight = m_nHeight;
    
    if( m_groundBuffer != NULL )
     delete [] m_groundBuffer;
    m_groundBuffer = new float[newBufSize];
    
    if( m_averageBuffer != NULL )
     delete [] m_averageBuffer;
    m_averageBuffer = new float[newBufSize];
    
    if( m_finalBuffer != NULL )
     delete [] m_finalBuffer;
    m_finalBuffer = new float[newBufSize];
    
    for( int i=0; i<newBufSize; i++ ) {
      m_groundBuffer[i] = 0.0;
      m_averageBuffer[i] = 0.0;
    }
    
    m_nBufSize = newBufSize;
  }  
}

// --------------------------- RunThread ------------------------------------
//
void DlgMainView::RunThread()
{
  INT dummy;
  char *pLast = NULL, *pMem = NULL, *pCurrent = NULL;

  
  m_bThreadRun = TRUE;
  while ( m_bThreadRun )
  {
    msleep(100);
    if ( m_bLive && m_hCamera)
    {
      m_bReady = TRUE;
      
      is_CaptureVideo( m_hCamera, IS_WAIT );
      // get last sequence buffer      
      is_GetActSeqBuf(m_hCamera, &dummy, &pMem, &pLast );
      pCurrent = pMem;

      while ( m_bLive && m_bThreadRun && m_hCamera )
      {
        if ( is_WaitEvent( m_hCamera, IS_SET_EVENT_FRAME, 100 ) == IS_SUCCESS )
        {
          if ( !m_bThreadRun )
          {
            is_StopLiveVideo( m_hCamera, IS_WAIT );
            return;
          }
          is_GetActSeqBuf(m_hCamera, &dummy, &pMem, &pLast );
          m_pLastBuffer = pLast;
          
          // Re-allocate buffers if settings have changed.
          resetVideo();
          
          // Copy image into float-space, apply buffers;
          int nNum = _GetImageNum( m_pLastBuffer );
          is_LockSeqBuf( m_hCamera, nNum, m_pLastBuffer );
          int bufIndex = 0;
          int i = 0;
          float contFactor = pow(1.1, spinContrast->value());
          float brtFactor = (spinBrightness->value() * 5);
          float finalScale = contFactor * 256.0;
          float finalOffset = contFactor * brtFactor;
          for( int iy=0; iy<m_nHeight; iy++ ) {
            for( int ix=0; ix<m_nWidth; ix++ ) {
              float cVal = (uchar)m_pLastBuffer[i] / 256.0;
              if( m_nAvgConstant != 1.0 ) {
                m_averageBuffer[i] = (cVal * m_nAvgConstant) + (m_averageBuffer[i] * (1.0 - m_nAvgConstant));
                if( checkSubtractGround->isChecked() ) {
                  m_finalBuffer[i] = m_averageBuffer[i] - m_groundBuffer[i];
                }
                else {
                  m_finalBuffer[i] = m_averageBuffer[i];
                }
              }
              else {
                m_averageBuffer[i] = cVal;
                if( checkSubtractGround->isChecked() ) {
                  m_finalBuffer[i] = cVal - m_groundBuffer[i];
                }
                else {
                  m_finalBuffer[i] = cVal;
                }
              }
              
//              float finalVal =  (((m_finalBuffer[i] * 256) + (spinBrightness->value() * 5)) * pow(1.1, spinContrast->value()));
              float finalVal =  (m_finalBuffer[i] * finalScale) + finalOffset;
              // use absolute value if checked
              if( checkAbsoluteValue->isChecked() ) {
                finalVal = fabs(finalVal);
              }
              
              // Make sure value is within displayable range
              if( finalVal > 255.0 ) {
                finalVal = 255.0;
              }
              if( finalVal < 0.0 )
                finalVal = 0.0;
              m_displayBuffer[i] = (char)(int) finalVal;
              i++;
              bufIndex++;
            }
            bufIndex += 0;
          }
          is_UnlockSeqBuf( m_hCamera, nNum, m_pLastBuffer );
          int frameTime = clock.elapsed();
          videoArea->newVideoFrame((uchar*)m_displayBuffer, m_nPosX, m_nPosY, m_nWidth, m_nHeight, m_nBitsPP, m_nColorMode);
          
          if( m_bSetGroundFrame ) {
            m_bSetGroundFrame = false;
            for( int i=0; i<m_nWidth * m_nHeight * m_nBitsPP / 8; i++ ) {
              m_groundBuffer[i] = m_finalBuffer[i];
            }
          }
          // Calculate framerate
          int msec = lastFrameTime.restart();
          char buf[100];
          sprintf(buf, "%0.2f frames/sec", 1000.0 / (float)msec);
          labelFramerate->setText(QString(buf));
          

          // send finalBuffer over to plot thread and get a new buffer to use
          Frame *curFrame = new Frame(m_finalBuffer, m_nWidth, m_nHeight, frameTime, 1);
          curFrame->contrast = spinContrast->value();
          curFrame->brightness = spinBrightness->value();
          Frame newFrame = plotThread->swapNewFrame(*curFrame);
          if( newFrame.buf == NULL ) {
            m_finalBuffer = new float[m_nWidth * m_nHeight * m_nBitsPP / 8];
/*            printf( "Image thread: plot thread took frame, making a new one\n" );*/
          }
          else {
            if( newFrame.height != m_nHeight || newFrame.width != m_nWidth ) {
              delete [] newFrame.buf;
              m_finalBuffer = new float[m_nWidth * m_nHeight * m_nBitsPP / 8];
/*              printf( "Image thread: frame returned by plot thread is unusable; deleting and making a new one.\n" );*/
            }
            else {
              m_finalBuffer = newFrame.buf;
/*              printf( "Image thread: reusing frame unused by plot thread\n" );*/
            }
          }


          if ( m_bReady )
          {
            m_bReady = FALSE;
            update();
          }
          //printf("Frame received\n");
        }
      }
      is_StopLiveVideo( m_hCamera, IS_WAIT );
    }
  }
}

void DlgMainView::OnSpinAverageConstant(double n)
{
  m_nAvgConstant = 1.0 - n;
  spinAverageConstant->setLineStep((1.0-n)/10.0);
}

void DlgMainView::OnPushStoreGroundFrame() {
  m_bSetGroundFrame = true;
}

void DlgMainView::OnCheckShowPlotRegion(bool b)
{
  videoArea->setPlotRegionShown(b);
}

void DlgMainView::OnCheckShowCropRegion(bool b)
{
  videoArea->setCropRegionShown(b);
}

void DlgMainView::OnPlotRegionChange() {
  
  
  char text[512];
  QRect r = videoArea->plotRegionRect();
  QRect c = videoArea->cropRegionRect();
  QPoint p = r.topLeft()-c.topLeft();
  QSize s = r.size();
  float a = videoArea->plotRegionAngle();
  
  sprintf(text, "x: %d y: %d\nw: %d h: %d\nangle: %0.2f", p.x(), p.y(), s.width(), s.height(), a * R2D);
  textPlotRegion->setText(QString(text));
  
  // Send new plot region data to plotThread
  plotThread->setRegion(RectRegion(p.x(), p.y(), s.width(), s.height(), a));
}


void DlgMainView::OnActiveRegionChange() {
  char text[512];
  QRect r = videoArea->cropRegionRect();
  QPoint p = r.topLeft();
  QSize s = r.size();
  
  sprintf(text, "x: %d y: %d\nw: %d h: %d", p.x(), p.y(), s.width(), s.height());
  textCropRegion->setText(QString(text));
}

void DlgMainView::OnBtnUpdateCropRegion() {
  updateCropRegion();
}

void DlgMainView::updateCropRegion() {
  QRect r = videoArea->cropRegionRect();
  QPoint p = r.topLeft();
  QSize s = r.size();

  // width must be divisible by 16, height must be divisible by 2
  int nearWidth = (s.width() / 16) * 16;
  int nearHeight = (s.height() / 2) * 2;
  int nearX = (p.x() / 16) * 16;
  int nearY = (p.y() / 2) * 2;
//   printf("Widget is %dx%d +%d, %d, nearest is %dx%d +%d, %d\n", s.width(), s.height(), p.x(), p.y(), nearWidth, nearHeight, nearX, nearY);
  // Set new size
  is_SetImageSize(m_hCamera, nearWidth, nearHeight);
  is_SetImagePos(m_hCamera, nearX, nearY);

  
  // Re-read to see actual size given
  int nxPos, nyPos, nwidth, nheight;
  nxPos  = is_SetImagePos( m_hCamera, IS_GET_IMAGE_POS_X, 0 );
  nyPos  = is_SetImagePos( m_hCamera, IS_GET_IMAGE_POS_Y, 0 );
  nwidth = is_SetImageSize( m_hCamera, IS_GET_IMAGE_SIZE_X, 0 );
  nheight= is_SetImageSize( m_hCamera, IS_GET_IMAGE_SIZE_Y, 0 );
//   printf("Requested %dx%d +%d,%d    Got %dx%d +%d,%d\n", nearWidth, nearHeight, nearX, nearY, nwidth, nheight, nxPos, nyPos);

    // Setting these variables should cause buffers to be reallocated..
  bool live = m_bLive;
  bool ready = m_bReady;
  m_bLive = FALSE;
  m_bReady = FALSE;
  msleep(100);
  m_nPosX = nxPos;
  m_nPosY = nyPos;
  m_nWidth = nwidth;
  m_nHeight = nheight;
  _EmptyImages();
  _AllocImages();
  videoArea->setCropRegion(QRect(QPoint(m_nPosX, m_nPosY), QSize(m_nWidth, m_nHeight)));
//  pixmapImage->update();

  _EmptyImages();
  m_bLive = live;
  m_bReady = ready;
  OnPlotRegionChange();
}

void DlgMainView::OnRadioPlotTypeChange()
{
  if( radioPlotType2D->isChecked() ) {
    plotArea->setDrawMode(0);
  }
  else if( radioPlotType3D->isChecked() ) {
    plotArea->setDrawMode(1);
  }
  else if( radioPlotTypeOscilloscope->isChecked() ) {
    plotArea->setDrawMode(2);
  }
  
}

void DlgMainView::OnCheckPlotInput()
{
  plotArea->setShowInput(checkPlotShowInput->isChecked());
}

void DlgMainView::OnCheckPlotWireframe() {
  plotArea->setWireframe( checkPlotWireframe->isChecked() );
}


void DlgMainView::OnCheckPlotUseImage() {
  plotArea->setUseImage( checkPlotUseImage->isChecked() );
}


void DlgMainView::OnCheckPlotLight() {
  plotArea->setLight( checkPlotLight->isChecked() );
}

void DlgMainView::OnSpinPlotLightAngleChanged() {
  plotArea->setLightPos(spinPlotLightAngle->value());
}

void DlgMainView::OnBtnSavePlot()
{
  QString s = QFileDialog::getSaveFileName(
    "",
    "Comma Separated Values (*.csv)",
    this,
    "save file dialog",
    "Choose a filename to save under" 
  );
  plotArea->savePlot(s, false);
  
}

void DlgMainView::OnBtnSaveImage() {
  QString s = QFileDialog::getSaveFileName(
    "", 
    "Image (*.png)", 
    this, 
    "save file dialog", 
    "Choose a filename to save under"
  );
  videoArea->saveImage(s);
}

void DlgMainView::OnBtnComputeScale() {
  
  // Grab plot data and estimated tick width
  int width;
  float* plotAvg = plotArea->getPlotAvg(&width);
  int estInterval = spinScaleEstimate->value();
  
  // Calculate mid curve, display in plot window
  float* mid = arrayMid(plotAvg, width, estInterval);
  plotArea->draw2DOverlay(mid, width);
  
  // Calculate ticks
  float *ticks;
  int numTicks;
  calcPlotCrossing(plotAvg, mid, width, &ticks, &numTicks);
//  printf("Counted %d ticks, %f wide(%f - %f)\n", numTicks, ticks[numTicks-1] - ticks[0], ticks[numTicks-1], ticks[0]);
  // display ticks in video frame and plot frame
/*  videoArea->drawTickOverlay(ticks, numTicks);*/
  if( numTicks < 2 ) {
    return;
  }
  // calculate average tick width, set scale
  spinPixelsPerUnit->setValue((ticks[numTicks-1] - ticks[0]) / (float)(numTicks-1));
}

void DlgMainView::OnSpinContrast(double val) {
//   printf("Set b step: %f\n", 1.0/pow(1.1, val));
  spinBrightness->setLineStep(1.0/pow(1.1, val));
}

void DlgMainView::OnSpinOscFrames(int n) {
  plotArea->resizeOscilloscope(n);
}

void DlgMainView::OnSpinPPUChange(double ppu) {
  plotArea->setPPU(ppu);
}

void DlgMainView::OnBtnScaleSensorDims() {
  char* sn = m_SensorInfo.strSensorName;
  if( strcmp(sn, "UI141x-M") == 0 ) {
    spinPixelsPerUnit->setValue(640.0/6.0);
  }
  else {
    
    QMessageBox::information ( 0, "Bummer.", QString("Unknown sensor type: ") + QString(sn), 0 );
  }

}

void DlgMainView::onSpinDownsample(int ds)
{
  plotThread->setDownsample(ds);
}

void DlgMainView::OnCheckPlotDisplay(bool s)
{
  plotArea->setDrawEnabled(s);
}

void DlgMainView::OnCheckPlotStore(bool s)
{
  plotArea->setStoreEnabled(s);
}

void DlgMainView::OnCheckPlotShowGrid(bool s)
{
  plotArea->setShowGrid(s);
}










#include "dlgmainview.moc"





