/***************************************************************************
 *   Copyright (C) 2005                                                    *
 *   Luke Campagnola                                                       *
 *   Colorado School of Mines                                              *
 *   lcampagn@mines.edu                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef DLGMAINVIEW_H
#define DLGMAINVIEW_H

#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qstring.h>
#include <qcolor.h>
#include <qdatetime.h>
#include <qradiobutton.h> 
#include <qgroupbox.h> 
#include <qlayout.h> 
#include <qthread.h> 
#include <qwhatsthis.h>
#include <knuminput.h>
// #include "vectormath.h"

#include "MainView.h"
#include "PlotThread.h"
#include "GLPlot.h"
#include "VideoDisplayArea.h"

#include <uEye.h>


#define IMAGE_COUNT    5
typedef struct _UEYE_IMAGE
{
  char  *pBuf;
  INT    nImageID;
  INT    nImageSeqNum;
  INT  nBufferSize;
} UEYE_IMAGE, *PUEYE_IMAGE;



class Preview : public QLabel, public QFilePreview
{
public:
  Preview( QWidget *parent=0 ) : QLabel( parent ) {}

  void previewUrl( const QUrl &u )
  {
    QString path = u.path();
    QPixmap pix( path );
    if ( pix.isNull() )
      setText( "This is not a pixmap" );
    else
      setPixmap( pix );
  }
};




class ImageThread : public QThread 
{
    public:
      virtual void RunThread() = 0;
        void run() { RunThread(); }
};


class EventThread : public QThread 
{
    public:
      virtual void RunEventThread() = 0;
        void run() { RunEventThread(); }
};


class DlgMainView : public MainView, ImageThread, EventThread
{
  Q_OBJECT

public:
  DlgMainView(QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
  ~DlgMainView();
  /*$PUBLIC_FUNCTIONS$*/

public slots:
  /*$PUBLIC_SLOTS$*/
  virtual void          OnRadioColorChange(int state);
  virtual void          OnBtnLive();
  virtual void          OnBtnProperties();  
  virtual void          OnBtnSaveToFile();
  virtual void          OnBtnExituEye();
  virtual void          OnBtnSnapshot();
  virtual void          OnBtnInituEye();
  virtual void          OnCheckUsememoryBoard();
  virtual void OnPushStoreGroundFrame();
  virtual void OnCheckShowPlotRegion(bool);
  virtual void OnCheckShowCropRegion(bool);
  virtual void OnPlotRegionChange();
  virtual void OnActiveRegionChange();
  virtual void OnBtnUpdateCropRegion();
    virtual void OnBtnSaveImage();
    virtual void OnBtnSavePlot();
    virtual void OnSpinPlotLightAngleChanged();
    virtual void OnSpinContrast(double);
    virtual void OnCheckPlotWireframe();
    virtual void OnCheckPlotUseImage();
    virtual void OnCheckPlotLight();
    virtual void OnCheckPlotInput();
    virtual void OnRadioPlotTypeChange();
    virtual void OnSpinAverageConstant(double);
    virtual void OnSpinOscFrames(int);
    virtual void OnBtnComputeScale();
    virtual void OnBtnScaleSensorDims();
    virtual void OnSpinPPUChange(double);
    virtual void OnCheckPlotShowGrid(bool);
    virtual void OnCheckPlotStore(bool);
    virtual void OnCheckPlotDisplay(bool);
    virtual void onSpinDownsample(int);

protected:
  /*$PROTECTED_FUNCTIONS$*/
  void timerEvent( QTimerEvent *e );
  void paintEvent ( QPaintEvent *e );
  void closeEvent ( QCloseEvent * e );
  void StartLivemode();

public:
  void RunThread();
  void RunEventThread();
  void RunPlotThread();
  
  void DrawImage( char *pBuffer);      // draw the captured image
  INT _GetImageNum( char* pbuf );      // get image num
  void _EmptyImages();          // empty images
  INT _GetImageID( char* pbuf );      // get image id
  BOOL _AllocImages();          // alloc images
 private:  
  void resetVideo();
  void updateCropRegion();
  
  VideoDisplayArea* videoArea;
  GLPlot* plotArea;

  // frame memory
  UEYE_IMAGE  m_Images[IMAGE_COUNT];  
 
  // Camera handle
  HIDS m_hCamera;
  // LiveVideo flag
  BOOL m_bLive;


  // Camerainfo
  CAMINFO m_CamInfo;
  // SensorInfo
  SENSORINFO m_SensorInfo;
  // Window size
  int m_nWidth;
  int m_nHeight;
  int m_nPosX, m_nPosY;
  int m_nBitsPP;
  int m_nBufSize;

  // Filter parameters
  float m_nAvgConstant;


  // thread terminate flag
  BOOL m_bThreadRun;
  
  BOOL m_bReady;
  char *m_pLastBuffer;
  float *m_groundBuffer;
  float *m_averageBuffer;
  float *m_finalBuffer;
  char *m_displayBuffer;
  int m_nDisplayWidth, m_nDisplayHeight;
  bool m_bSetGroundFrame;
  
  // run event thread
  BOOL m_bRunEventThread;

  PlotThread* plotThread;
  
  // used color mode
  int m_nColorMode;
  
  // track framerate
  QTime lastFrameTime;
  QTime clock;
};

#endif





