/***************************************************************************
 *   Copyright (C) 2005                                                    *
 *   Luke Campagnola                                                       *
 *   Colorado School of Mines                                              *
 *   lcampagn@mines.edu                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "dlgproperties.h"


// -------------------  Constructor  -----------------------
//
DlgProperties::DlgProperties(HIDS hCam, CAMINFO camInfo, SENSORINFO sensorInfo, QWidget* parent, const char* name, bool modal, WFlags fl)
: _DlgProperties(parent,name, modal,fl)
{
	QString strTemp, strTemp1;
	
	m_bInit = TRUE;
	m_hCamera = hCam;
 	memcpy( &m_CamInfo, &camInfo, sizeof(camInfo) );
	memcpy( &m_SensorInfo, &sensorInfo, sizeof(sensorInfo) );
	
	
	// brightness
	sliderBrightness->setMinValue( IS_MIN_BRIGHTNESS );
	sliderBrightness->setMaxValue( IS_MAX_BRIGHTNESS );
	spinBoxBrightness->setMinValue( IS_MIN_BRIGHTNESS );
	spinBoxBrightness->setMaxValue( IS_MAX_BRIGHTNESS );
		
	// contrast
	sliderContrast->setMinValue( IS_MIN_CONTRAST );
	sliderContrast->setMaxValue( IS_MAX_CONTRAST );
	spinBoxContrast->setMinValue( IS_MIN_CONTRAST );
	spinBoxContrast->setMaxValue( IS_MAX_CONTRAST );

	// gamma
	sliderGamma->setMinValue( IS_MIN_GAMMA );
	sliderGamma->setMaxValue( IS_MAX_GAMMA );
	spinBoxGamma->setMinValue( IS_MIN_GAMMA );
	spinBoxGamma->setMaxValue( IS_MAX_GAMMA );
		
	// pixelclock
	int minPclk, maxPclk;
	is_GetPixelClockRange(m_hCamera, &minPclk, &maxPclk );
	sliderPixelclock->setMinValue( minPclk );
	sliderPixelclock->setMaxValue( maxPclk );	
	spinPixelclock->setMinValue( minPclk );
	spinPixelclock->setMaxValue( maxPclk );
	
	// set text
	strTemp.setNum( minPclk, 10);
	strTemp1.setNum(maxPclk, 10);
	LabelPixelclock->setText( "Pixelclock (" + strTemp + " - " + strTemp1 + " MHz)" );

	m_pDlgMainView = (DlgMainView*)parent;
}



// -------------------  Destructor  -----------------------
//
DlgProperties::~DlgProperties()
{
}




// -------------------  showEvent  -----------------------
//
void DlgProperties::showEvent ( QShowEvent * )
{
	QString strTemp;
	BOOL bChecked;
	int nTemp = is_SetPixelClock( m_hCamera, IS_GET_PIXEL_CLOCK );
	
	m_bInit = FALSE;
	
	// Hardware gain	
	sliderMaster->setValue( is_SetHardwareGain( m_hCamera, IS_GET_MASTER_GAIN, 0, 0, 0 ) );
	sliderRed->setValue( is_SetHardwareGain( m_hCamera, IS_GET_RED_GAIN, 0, 0, 0 ) );
	sliderGreen->setValue( is_SetHardwareGain( m_hCamera, IS_GET_GREEN_GAIN, 0, 0, 0 ) );
	sliderBlue->setValue( is_SetHardwareGain( m_hCamera, IS_GET_BLUE_GAIN, 0, 0, 0 ) );
	if ( !m_SensorInfo.bMasterGain )
		sliderMaster->setEnabled(FALSE);
	if ( !m_SensorInfo.bRGain )
		sliderRed->setEnabled(FALSE);
	if ( !m_SensorInfo.bGGain )
		sliderGreen->setEnabled(FALSE);
	if ( !m_SensorInfo.bBGain )
		sliderBlue->setEnabled(FALSE);
	
	// bright, cont gamma
	sliderBrightness->setValue ( is_SetBrightness(m_hCamera, IS_GET_BRIGHTNESS) );	
	sliderContrast->setValue ( is_SetContrast(m_hCamera, IS_GET_CONTRAST) );
	sliderGamma->setValue ( is_SetGamma(m_hCamera, IS_GET_GAMMA) );
	
	// pixelclock, exposure
	sliderPixelclock->setValue( nTemp );
	
	// flip up down
	bChecked = is_SetRopEffect( m_hCamera, IS_GET_ROP_EFFECT, 0, 0 ) & IS_SET_ROP_MIRROR_UPDOWN;
	checkBoxUpDown->setChecked( bChecked );
	// color correction
	bChecked = is_SetColorCorrection( m_hCamera, IS_GET_CCOR_MODE, 0 ) == IS_CCOR_ENABLE;
	checkBoxColorCorr->setChecked( bChecked );
	// bad pixel
	bChecked = is_SetBadPixelCorrection( m_hCamera, IS_GET_BPC_MODE, 0 ) ==
			 IS_BPC_ENABLE_LEVEL_1;
	checkBoxBadPixel->setChecked( bChecked );
	// edge enhancement
	buttonGroupEdgeEnhancement->setButton(
	is_SetEdgeEnhancement(m_hCamera, IS_GET_EDGE_ENHANCEMENT) );

	if ( m_SensorInfo.nColorMode != IS_COLORMODE_BAYER )
	{
		buttonGroupWhitebalance->setEnabled(FALSE);
		checkBoxColorCorr->setEnabled(FALSE);
	}
	else
	{
		buttonGroupWhitebalance->setButton(
		is_SetWhiteBalance( m_hCamera, IS_GET_WB_MODE )   );
	}
	
	// image pos & size
	UpdateWindowSizeCtrls();

}



// -------------------  UpdateWindowSizeCtrls  -----------------------
//
void DlgProperties::UpdateWindowSizeCtrls()
{
	if ( m_bInit )
		return;
		
	// get size
	int xPos, yPos, width, height;
	
	xPos  = is_SetImagePos( m_hCamera, IS_GET_IMAGE_POS_X, 0 );
	yPos  = is_SetImagePos( m_hCamera, IS_GET_IMAGE_POS_Y, 0 );
	width = is_SetImageSize( m_hCamera, IS_GET_IMAGE_SIZE_X, 0 );
	height= is_SetImageSize( m_hCamera, IS_GET_IMAGE_SIZE_Y, 0 );
	

	m_bInit=TRUE;
	spinBoxX->setMinValue(0);	spinBoxX->setMaxValue(m_SensorInfo.nMaxWidth-width);
	spinBoxY->setMinValue(0);	spinBoxY->setMaxValue(m_SensorInfo.nMaxHeight-height);
	spinBoxWidth->setMinValue(0);	spinBoxWidth->setMaxValue(m_SensorInfo.nMaxWidth-xPos);
	spinBoxHeight->setMinValue(0);	spinBoxHeight->setMaxValue(m_SensorInfo.nMaxHeight-yPos);
	m_bInit=FALSE;
	
	spinBoxX->setValue ( xPos );
	spinBoxY->setValue ( yPos );
	spinBoxWidth->setValue( width );
	spinBoxHeight->setValue( height );
}




// -------------------  UpdateFramerate  -----------------------
//
void DlgProperties::UpdateFramerate()
{
	if ( m_bInit )
		return;
		
	double minTime = 0, maxTime = 0, intervall = 0, framerate = 0;
	QString strTemp, strTemp1;

	// exposure
	is_GetFrameTimeRange( m_hCamera, &minTime, &maxTime, &intervall );	
	is_SetFrameRate( m_hCamera, IS_GET_FRAMERATE, &framerate );
	m_bInit=TRUE;
	sliderFramerate->setMinValue( (int)(1/maxTime * 100) );
	sliderFramerate->setMaxValue( (int)(1/minTime * 100) );
	sliderFramerate->setValue( (int)(framerate*100) );
	m_bInit=FALSE;
	
	strTemp.setNum(framerate, 'f', 2 );	
	EditFramerate->setText( strTemp );	

	strTemp.setNum( 1/maxTime, 'f', 2);
	strTemp1.setNum( 1/minTime, 'f', 2);
	LabelFramerate->setText( "Framerate (" + strTemp + " - " + strTemp1 + " )" );
	
	UpdateExposure();
}




// -------------------  UpdateExposure  -----------------------
//
void DlgProperties::UpdateExposure()
{
	if ( m_bInit )
		return;

	double minExp = 0, maxExp = 0, intervall = 0, exposure = 0;
	QString strTemp, strTemp1;

	// exposure
	is_GetExposureRange( m_hCamera, &minExp, &maxExp, &intervall );	
	is_SetExposureTime( m_hCamera, IS_GET_EXPOSURE_TIME, &exposure );
	m_bInit=TRUE;
	sliderExposure->setMinValue( (int)(minExp * 100) );
	sliderExposure->setMaxValue( (int)(maxExp * 100) );
	sliderExposure->setValue( (int)(exposure*100) );
	m_bInit=FALSE;

	strTemp.setNum(exposure, 'f', 2 );	
	EditExposure->setText( strTemp );	

	strTemp.setNum( minExp, 'f', 2);
	strTemp1.setNum( maxExp, 'f', 2);
	LabelExposure->setText( "Exposure time (" + strTemp + " - " + strTemp1 + " ms)" );
}




// -------------------  OnButtonDefault  -----------------------
//
void DlgProperties::OnButtonDefault()
{
	is_SetGamma( m_hCamera, IS_DEFAULT_GAMMA );
	sliderGamma->setValue( is_SetGamma( m_hCamera, IS_GET_GAMMA ) );
	
	is_SetBrightness( m_hCamera, IS_DEFAULT_BRIGHTNESS );
	sliderBrightness->setValue( is_SetBrightness( m_hCamera, IS_GET_BRIGHTNESS ) );
	
	is_SetContrast( m_hCamera, IS_DEFAULT_CONTRAST );
	sliderContrast->setValue( is_SetContrast( m_hCamera, IS_GET_CONTRAST ) );
	
	sliderMaster->setValue( 0 );
	sliderRed->setValue( 0 );
	sliderGreen->setValue( 0 );
	sliderBlue->setValue( 0 );
}


// -------------------  OnSliderChangeGamma  -----------------------
//
void DlgProperties::OnSliderChangeGamma(int value)
{
	if ( m_bInit )
		return;

	spinBoxGamma->setValue( value );
	sliderGamma->setValue( value );	
	is_SetGamma( m_hCamera, value );
}


// -------------------  OnSliderChangeCont  -----------------------
//
void DlgProperties::OnSliderChangeCont(int value)
{
	if ( m_bInit )
		return;

	spinBoxContrast->setValue( value );
	sliderContrast->setValue( value );	
	is_SetContrast( m_hCamera, value );	
}


// -------------------  OnSliderChangeBright  -----------------------
//
void DlgProperties::OnSliderChangeBright(int value)
{
	if ( m_bInit )
		return;

	spinBoxBrightness->setValue( value );
	sliderBrightness->setValue( value );
	is_SetBrightness( m_hCamera, value );
}




// -------------------  OnSliderChangeMaster  -----------------------
//
void DlgProperties::OnSliderChangeMaster(int value)
{
	if ( m_bInit )
		return;

	spinBoxMaster->setValue( value );
	sliderMaster->setValue( value );
	is_SetHardwareGain(m_hCamera, value, IS_IGNORE_PARAMETER, IS_IGNORE_PARAMETER, IS_IGNORE_PARAMETER );	
}



// -------------------  OnSliderChangeBlue  -----------------------
//
void DlgProperties::OnSliderChangeBlue(int value)
{
	if ( m_bInit )
		return;

	spinBoxBlue->setValue( value );
	sliderBlue->setValue( value );
	is_SetHardwareGain(m_hCamera, IS_IGNORE_PARAMETER, IS_IGNORE_PARAMETER, IS_IGNORE_PARAMETER, value );	
}



// -------------------  OnSliderChangeGreen  -----------------------
//
void DlgProperties::OnSliderChangeGreen(int value)
{
	if ( m_bInit )
		return;

	spinBoxGreen->setValue( value );
	sliderGreen->setValue( value );
	is_SetHardwareGain(m_hCamera, IS_IGNORE_PARAMETER, IS_IGNORE_PARAMETER, value, IS_IGNORE_PARAMETER );
}



// -------------------  OnSliderChangeRed  -----------------------
//
void DlgProperties::OnSliderChangeRed(int value)
{
	if ( m_bInit )
		return;

	spinBoxRed->setValue( value );
	sliderRed->setValue( value );
	is_SetHardwareGain(m_hCamera, IS_IGNORE_PARAMETER, value, IS_IGNORE_PARAMETER, IS_IGNORE_PARAMETER );
}







//****************************************************************************************
//************************************* TIMING *******************************************
//
//
// -------------------  OnSliderChangePixelClock  -----------------------
//
void DlgProperties::OnSliderChangePixelClock(int value)
{
	if ( m_bInit )
		return;

	is_SetPixelClock(m_hCamera, value );
	spinPixelclock->setValue( value );
	//sliderPixelclock->setValue( value );
	UpdateFramerate();
}



// -------------------  OnSliderChangePixelClock  -----------------------
//
void DlgProperties::OnSliderChangeExposure(int value)
{
	if ( m_bInit )
		return;

	double expTime 		= 0;
	double newexpTime	= 0;
	expTime 		= (float)value / 100;
		
	is_SetExposureTime(m_hCamera, expTime, &newexpTime );	
		
	UpdateExposure();	
}


// -------------------  OnSliderChangeFrameRate  -----------------------
//
void DlgProperties::OnSliderChangeFrameRate(int value)
{
	if ( m_bInit )
		return;

//	QString strTemp;
	double frameRate 	= 0;
	double newframeRate	= 0;
	frameRate 		= (float)value / 100;
	
	is_SetFrameRate(m_hCamera, frameRate, &newframeRate );
		
	UpdateFramerate();	
}




//****************************************************************************************
//************************************* Proccessing **************************************
//
void DlgProperties::OnButtonUpDown()
{
	BOOL bEnable = checkBoxUpDown->isChecked();
	is_SetRopEffect( m_hCamera, IS_SET_ROP_MIRROR_UPDOWN, bEnable, 0);
}

void DlgProperties::OnButtonEnableColorCor()
{
	int nMode =  checkBoxColorCorr->isChecked() ? IS_CCOR_ENABLE : IS_CCOR_DISABLE;
	is_SetColorCorrection( m_hCamera, nMode, 0 );	
}

void DlgProperties::OnButtonBadPixel()
{//	IS_EDGE_EN_DISABLE		0
//	IS_EDGE_EN_STRONG		1
//	IS_EDGE_EN_WEAK			2	

	INT nMode = checkBoxBadPixel->isChecked() ? IS_BPC_ENABLE_LEVEL_1 : IS_BPC_DISABLE;
	is_SetBadPixelCorrection( m_hCamera, nMode, 0 );
}

void DlgProperties::OnRadioEdgeEnhancement()
{
//	IS_EDGE_EN_DISABLE		0
//	IS_EDGE_EN_STRONG		1
//	IS_EDGE_EN_WEAK			2	
	is_SetEdgeEnhancement(m_hCamera, buttonGroupEdgeEnhancement->selectedId() );
}



void DlgProperties::OnRadioWhiteBalance()
{
	is_SetWhiteBalance( m_hCamera, buttonGroupWhitebalance->selectedId() );
}





//****************************************************************************************
//************************************* Proccessing **************************************
//
void DlgProperties::OnSpinBoxSizeChanged(int value)
{
	if ( m_bInit )
		return;

	is_SetImageSize( m_hCamera, spinBoxWidth->value(), spinBoxHeight->value() );
	UpdateWindowSizeCtrls();
	m_pDlgMainView->_EmptyImages();
}


void DlgProperties::OnSpinBoxPosChanged(int value)
{
	if ( m_bInit )
		return;

	is_SetImagePos( m_hCamera, spinBoxX->value(), spinBoxY->value() );
	UpdateWindowSizeCtrls();
	m_pDlgMainView->_EmptyImages();
}


#include "dlgproperties.moc"












